CREATE OR REPLACE PROCEDURE RQDROP_VIEW
(id integer) is
  cid integer;
  sql_txt1 varchar2(50);
  sql_txt2 varchar2(50);
  view_not_found EXCEPTION;
  PRAGMA EXCEPTION_INIT(view_not_found, -942);
BEGIN
  sql_txt1 := 'DROP VIEW RQATTRVALUESVIEW_' || id;
  sql_txt2 := 'DROP VIEW RQATTRVALUESSORTVIEW_' || id;
  cid := DBMS_SQL.OPEN_CURSOR;
  DBMS_SQL.PARSE(cid, sql_txt1, dbms_sql.v7);
  DBMS_SQL.PARSE(cid, sql_txt2, dbms_sql.v7);
  DBMS_SQL.CLOSE_CURSOR(cid);
EXCEPTION
  WHEN view_not_found THEN
    DBMS_SQL.CLOSE_CURSOR(cid);
END;
/
CREATE OR REPLACE PROCEDURE RQDROP_VIEWS
(projId integer) is 
  attrId integer;
  CURSOR c1 (proj_id integer) IS
    SELECT RTF.UserDefinedFieldId
    FROM RqRequirementTypeFields RTF,
         RqProjectRequirementTypes PRT
    WHERE RTF.RequirementTypeId = PRT.RequirementTypeId
    AND PRT.ProjectId = proj_id;
BEGIN
  OPEN c1(projId);
  LOOP
    FETCH c1 INTO attrId;
    EXIT WHEN c1%NOTFOUND;
    RqDrop_View(attrId);
  END LOOP;
  CLOSE c1;
END;
/
CREATE OR REPLACE TRIGGER RQDELETE_PROJECT_BLOCK 
BEFORE DELETE ON RqProjects
FOR EACH ROW
BEGIN
  if :old.versionreason != '@DELETING@' then 
     raise_application_error(-20001, 'Use the RQDELETE_PROJECT stored procedure to delete projects.', FALSE);
  end if;
EXCEPTION
  WHEN others THEN
    RAISE;
END;
/
CREATE OR REPLACE PROCEDURE RQDELETE_PROJECT
(projId integer) is
	projectname	rqprojects.name%TYPE;
BEGIN
  -- Get the project name
  BEGIN
	SELECT name 
	INTO   projectname
	FROM   rqprojects
	WHERE  id = projId;
  EXCEPTION
        WHEN OTHERS THEN -- projectname := 'Unknown';
	     raise_application_error(-20002, 'Project ' || projId || ' does not exist.', FALSE);
  END;

  dbms_output.put_line('Deleting project ' || TO_CHAR(projId) || ' (' || projectname || ')');

  RQDROP_VIEWS(projId);

  DELETE FROM RQDOCUMENTTYPEUSERS
  WHERE PROJECTID = projId;

  DELETE FROM RQREQUIREMENTTYPEUSERS
  WHERE PROJECTID = projId;

  DELETE FROM RQUSERDEFINEDFIELDUSERS
  WHERE PROJECTID = projId;

  DELETE FROM RQUSERDEFINEDLISTITEMUSERS
  WHERE PROJECTID = projId;

  DELETE FROM RQUSERS
  WHERE GROUPID IN
     (SELECT USERGROUPID
     FROM RQPROJECTUSERS
     WHERE PROJECTID = projId);

  DELETE FROM RQUSERGROUPS
  WHERE ID IN
     (SELECT USERGROUPID
     FROM RQPROJECTUSERS
     WHERE PROJECTID = projId);

  DELETE RQDISCUSSIONS
  WHERE ROOTID IN
     (SELECT DISCUSSIONID
     FROM RQPROJECTDISCUSSIONS
     WHERE PROJECTID = projId);

  DELETE RQOVERFLOWHISTORY
  WHERE OVERFLOWTYPE = 2
  AND OVERFLOWID IN
     (SELECT DOCUMENTID
     FROM RQPROJECTDOCUMENTS
     WHERE PROJECTID = projId);

  DELETE RQDOCUMENTS
  WHERE ID IN
     (SELECT DOCUMENTID
     FROM RQPROJECTDOCUMENTS
     WHERE PROJECTID = projId);

  DELETE RQDOCUMENTTYPES
  WHERE ID IN
     (SELECT DOCUMENTTYPEID
     FROM RQPROJECTDOCUMENTTYPES
     WHERE PROJECTID = projId);

  DELETE RQTORELATIONSHIPS
  WHERE EXTERNALPROJECTID = 0
  AND REQUIREMENTID IN
     (SELECT REQUIREMENTID
     FROM RQPROJECTREQUIREMENTS
     WHERE PROJECTID = projId);

  DELETE RQTORELATIONSHIPS
  WHERE EXTERNALPROJECTTOID = 0
  AND TOID IN
     (SELECT REQUIREMENTID
     FROM RQPROJECTREQUIREMENTS
     WHERE PROJECTID = projId);

  DELETE RQTORELATIONSHIPS
  WHERE EXTERNALPROJECTID = projId
  OR EXTERNALPROJECTTOID = projId;

  DELETE RQOVERFLOWHISTORY
  WHERE OVERFLOWTYPE = 1
  AND OVERFLOWID IN
     (SELECT REQUIREMENTID
     FROM RQPROJECTREQUIREMENTS
     WHERE PROJECTID = projId);

  DELETE RQREQUIREMENTS
  WHERE ID IN
     (SELECT REQUIREMENTID
     FROM RQPROJECTREQUIREMENTS
     WHERE PROJECTID = projId);

  DELETE RQUSERDEFINEDFIELDS
  WHERE ID IN
     (SELECT USERDEFINEDFIELDID
     FROM RQREQUIREMENTTYPEFIELDS F, RQPROJECTREQUIREMENTTYPES P
     WHERE F.REQUIREMENTTYPEID = P.REQUIREMENTTYPEID
     AND PROJECTID = projId);

  DELETE RQREQUIREMENTTYPES
  WHERE ID IN
     (SELECT REQUIREMENTTYPEID
     FROM RQPROJECTREQUIREMENTTYPES
     WHERE PROJECTID = projId);

  DELETE RQQUERYDEFINITIONS
  WHERE ID IN
     (SELECT SECONDARYQUERYID
     FROM RQVIEWS
     WHERE PROJECTID = projId);
  
  DELETE RQQUERYDEFINITIONS
  WHERE ID IN
     (SELECT QUERYID
     FROM RQVIEWS
     WHERE PROJECTID = projId);

  DELETE RQOVERFLOWHISTORY
  WHERE OVERFLOWTYPE = 3
  AND OVERFLOWID = projId;

  DELETE RQEXTERNALPROJECTS
  WHERE ID = projId;

  UPDATE RQPROJECTS SET versionreason = '@DELETING@' WHERE ID = projID;

  DELETE FROM RQPROJECTS
  WHERE ID = projId;

  COMMIT;

  DELETE RQEXTERNALPROJECTS
  WHERE ID NOT IN
     (SELECT EXTERNALPROJECTID
     FROM RQPROJECTEXTERNALPROJECTS);

  COMMIT;

  dbms_output.put_line('Delete successful.');

EXCEPTION
  WHEN others THEN
      ROLLBACK;
  RAISE;
END;
/





